import os
import re

import easyocr
import requests
from loguru import logger
from PIL import Image


def save_all_images_from_markdown(filename, outputdir):
    # 디렉토리가 존재하지 않으면 생성
    if not os.path.exists(outputdir):
        os.makedirs(outputdir)

    # 마크다운 파일 읽기
    with open(filename, "r", encoding="utf-8") as file:
        markdown = file.read()

    # 이미지 URL 찾기
    urls = re.findall(r"!\[.*?\]\((.*?)\)", markdown)

    # 이미지 저장 순서 번호
    image_number = 1
    for url in urls:
        # 파일 확장자 추출

        extension = os.path.splitext(url)[1]
        # 이미지 다운로드
        logger.info(f"download image:{url}")
        response = requests.get(url)
        if response.status_code == 200:
            # 이미지 파일 저장
            file_path = os.path.join(outputdir, f"{image_number}{extension}")
            with open(file_path, "wb") as f:
                f.write(response.content)
            image_number += 1
        else:
            print(f"Failed to download {url}")


def merge_all_images_from_directory(target_dir, save_fn):
    # 이미지 파일 목록을 가져온 후, 파일명 숫자 기준으로 정렬
    files = [
        os.path.join(target_dir, f)
        for f in os.listdir(target_dir)
        if f.endswith((".png", ".jpg", ".jpeg", ".bmp"))
    ]
    files.sort(key=lambda x: int(os.path.splitext(os.path.basename(x))[0]))

    # 이미지들을 열어서 리스트에 저장
    images = [Image.open(file) for file in files]

    # 가장 큰 너비 찾기
    max_width = max(image.width for image in images)

    # 각 이미지를 가장 큰 너비에 맞게 조정하면서 전체 높이 계산
    resized_images = []
    total_height = 0
    for image in images:
        # 가로세로 비율 유지하면서 너비를 max_width에 맞춤
        ratio = max_width / image.width
        new_height = int(image.height * ratio)
        resized_image = image.resize((max_width, new_height), Image.Resampling.LANCZOS)
        resized_images.append(resized_image)
        total_height += new_height

    # 새 이미지 생성
    new_im = Image.new("RGB", (max_width, total_height))

    # 이미지 합치기
    y_offset = 0
    for im in resized_images:
        new_im.paste(im, (0, y_offset))
        y_offset += im.height

    # 결과 이미지 저장
    new_im.save(save_fn)


def read_all_text_from_image(image_fn, save_fn):
    # EasyOCR Reader를 생성합니다. 사용할 언어를 설정할 수 있습니다. 여기서는 영어('en')로 설정합니다.
    reader = easyocr.Reader(["en", "ko"])

    # 이미지에서 텍스트를 추출합니다.
    results = reader.readtext(image_fn)

    # 추출된 텍스트를 저장할 문자열을 생성합니다.
    extracted_text = ""
    for bbox, text, prob in results:
        extracted_text += text + "\n"

    # 추출된 텍스트를 파일로 저장합니다.
    with open(save_fn, "w", encoding="utf-8") as file:
        file.write(extracted_text)
                      