import os
import subprocess
import tempfile
import time
from abc import ABC, abstractmethod
from urllib.parse import quote, urlparse

from loguru import logger


class ImageUploader(ABC):
    """이미지 업로드를 위한 추상 기본 클래스"""

    @abstractmethod
    def _upload_to_host(self, file_path: str) -> str:
        """
        호스팅 서비스에 이미지를 업로드합니다.

        Args:
            file_path (str): 업로드할 이미지 파일 경로

        Returns:
            str: 업로드된 이미지의 URL

        Raises:
            ImageUploadError: 업로드 실패시 발생
        """
        pass

    def upload(self, source: str, max_retries: int = 3) -> str:
        """
        이미지를 업로드하고 URL을 반환합니다.

        Args:
            source (str): 이미지 파일 경로 또는 URL
            max_retries (int): URL 다운로드 시 최대 재시도 횟수

        Returns:
            str: 업로드된 이미지의 URL

        Raises:
            ImageUploadError: 업로드 실패시 발생
        """
        if not source:
            logger.error("소스가 비어있습니다.")
            raise ImageUploadError("소스가 비어있습니다.")

        # URL인지 파일 경로인지 확인
        is_url = bool(urlparse(source).scheme)

        try:
            if is_url:
                logger.info(f"URL에서 이미지 업로드 시도: {source}")
                return self._upload_from_url(source, max_retries)
            else:
                logger.info(f"파일에서 이미지 업로드 시도: {source}")
                return self._upload_from_file(source)
        except Exception as e:
            logger.error(f"이미지 업로드 실패: {str(e)}")
            raise ImageUploadError(f"이미지 업로드 실패: {str(e)}")

    def _upload_from_file(self, file_path: str) -> str:
        """파일에서 직접 업로드"""
        if not os.path.exists(file_path):
            logger.error(f"파일을 찾을 수 없습니다: {file_path}")
            raise ImageUploadError(f"파일을 찾을 수 없습니다: {file_path}")

        logger.info(f"파일 업로드 중: {file_path}")
        return self._upload_to_host(file_path)

    def _upload_from_url(self, url: str, max_retries: int) -> str:
        """URL에서 이미지를 다운로드하고 업로드"""
        # URL 인코딩
        parsed_url = urlparse(url)
        encoded_path = quote(parsed_url.path)
        encoded_url = f"{parsed_url.scheme}://{parsed_url.netloc}{encoded_path}"

        # 임시 파일 생성
        file_ext = os.path.splitext(parsed_url.path)[1].lower() or ".jpg"
        temp_file = tempfile.NamedTemporaryFile(suffix=file_ext, delete=False)
        temp_path = temp_file.name
        temp_file.close()

        try:
            logger.info(f"이미지 다운로드 중: {encoded_url}")
            self._download_image(encoded_url, temp_path, max_retries)
            logger.info(f"임시 파일에서 업로드 중: {temp_path}")
            return self._upload_to_host(temp_path)
        finally:
            # 임시 파일 정리
            try:
                if os.path.exists(temp_path):
                    os.unlink(temp_path)
                    logger.info(f"임시 파일 삭제 완료: {temp_path}")
            except OSError as e:
                logger.warning(f"임시 파일 삭제 실패: {str(e)}")

    def _download_image(self, url: str, temp_path: str, max_retries: int) -> None:
        """이미지 다운로드"""
        curl_command = [
            "curl",
            "-L",
            "-f",
            "-S",
            "-s",
            "--connect-timeout",
            "30",
            "--max-time",
            "60",
            "-A",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64)",
            "-o",
            temp_path,
            url,
        ]

        for attempt in range(max_retries):
            try:
                logger.info(f"다운로드 시도 {attempt + 1}/{max_retries}: {url}")
                result = subprocess.run(curl_command, capture_output=True, text=True)
                if result.returncode == 0:
                    logger.info("다운로드 성공")
                    break

                if attempt == max_retries - 1:
                    logger.error(f"Curl 다운로드 실패: {result.stderr}")
                    raise ImageUploadError(f"Curl 다운로드 실패: {result.stderr}")

                time.sleep(2**attempt)
            except Exception as e:
                logger.error(f"이미지 다운로드 실패: {str(e)}")
                if attempt == max_retries - 1:
                    raise ImageUploadError(f"이미지 다운로드 실패: {str(e)}")
                continue

        if os.path.getsize(temp_path) < 100:
            logger.error("다운로드된 파일이 너무 작습니다")
            raise ImageUploadError("다운로드된 파일이 너무 작습니다")


class ImageUploadError(Exception):
    """이미지 업로드 관련 예외""" 